/*
 * GIMP Text Colorer -- This is a plug-in for The GIMP 1.1
 * Copyright (C) 2000 Marco Lamberto <lm@geocities.com>
 * Web page: http://www.geocities.com/marcolamberto/gimp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id$
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>
#include <libgimp/gimpintl.h>

#include "text_colorer.h"

#include "libgimp/stdplugins-intl.h"


typedef struct
{
	gchar filename[8192];
	gchar string[8192];
	gint font_size;
	gboolean use_string;
} TCStatus;

typedef struct
{
	GtkWidget *window;
	GtkWidget *button_browse;
	GtkWidget *file_sel;
	GtkWidget *file_name;
	GtkWidget *string;
	GtkObject *adj;
	GtkWidget *font_size;
	gboolean ok_pressed;
	TCStatus *status;
} TCMainWindow;


static void query(void);
static void run(char *name, gint nparams, GimpParam *param, gint *nreturn_vals, GimpParam **return_vals);
static gint32 save_image(gchar * filename, gint32 image_id, gint32 drawable_id);
static gboolean save_dialog(void);
static void on_ok_clicked(GtkWidget *widget, gpointer data);
static void on_file_sel_close(GtkWidget *widget, gpointer data);
static void on_browse_clicked(GtkWidget *widget, gpointer data);
static void on_file_choose(GtkWidget *widget, gpointer data);
static void on_string_choose(GtkWidget *widget, gpointer data);


GimpPlugInInfo PLUG_IN_INFO =
{
	NULL,
	NULL,
	query,
	run
};

TCMainWindow *mw = NULL;

static TCStatus tc_status =
{
	"",
	"",
	2,
	FALSE
};

guint xhtml_output = 0;

gchar *TITLE[] =
	{
		"CHTML",
		"CXHTML"
	};

gchar *HEAD[] =
	{
		"<HTML>\n"
		"<HEAD>\n"
		"	<TITLE>%s (%d x %d)</TITLE>\n"
		"	<META NAME=\"generator\" CONTENT=\"GIMP Text Colorer %s by Marco Lamberto - http://www.geocities.com/marcolamberto/gimp/ \">\n"
		"</HEAD>\n"
		"<BODY BGCOLOR=\"#%06X\">\n"
		"<FONT SIZE=\"%d\"><PRE>\n",

		"<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
		"<!DOCTYPE html PUBLIC\n"
		"  \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n"
		"  \"DTD/xhtml1-transaitional.dtd\">\n"
		"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n"
		"<head>\n"
		"	<title>%s (%d x %d)</title>\n"
		"	<meta name='generator' content='GIMP Text Colorer %s by Marco Lamberto - http://the.sunnyspot.org/gimp/' />\n"
		"</head>\n"
		"<body>\n\n"
		"<!-- cut here (image start) -->\n"
		"<pre style='background: #%06x; font-family: fixed,sans-serif; font-size: %dpx;'>"
	};

gchar *CBLOCK_START[] =
	{
		"<FONT COLOR=\"#%02X%02X%02X\">",
		"<span style='color: #%02x%02x%02x'>"
	};

gchar *CBLOCK_END[] =
	{
		"</FONT>",
		"</span>"
	};

gchar *TAIL[] =
	{
		"</FONT>\n"
		"</PRE></FONT>\n"
		"</BODY>\n"
		"</HTML>",

		"</span>"
		"</pre>\n"
		"<!-- cut here (image end) -->\n\n"
		"</body>\n"
		"</html>\n"
	};


MAIN()


static void
query(void)
{
	static GimpParamDef save_args[] =
	{
		{GIMP_PDB_INT32,		"run_mode",	"Interactive, non-interactive"},
		{GIMP_PDB_IMAGE,		"image",		"Input image"},
		{GIMP_PDB_DRAWABLE,	"drawable",	"Drawable to save"},
	
		{GIMP_PDB_STRING,		"filename",			"The name of the file to save the image in"},
		{GIMP_PDB_STRING,		"raw_filename",	"The name entered"},
	};
	static gint nsave_args = sizeof(save_args) / sizeof(save_args[0]);


	gimp_install_procedure("file_text_colorer_save",
		"FIXME",
		"",
		"Marco Lamberto <lm AT sunnyspot DOT org>",
		"Marco Lamberto",
		"16 Nov 2000",
		N_("<Save>/Color HTML (.chtml)"),
		"INDEXED",
		GIMP_PLUGIN,
		nsave_args, 0,
		save_args, NULL);

	gimp_register_save_handler("file_text_colorer_save", "chtml", "");

	gimp_install_procedure("file_text_colorer_save_xhtml",
		"FIXME",
		"",
		"Marco Lamberto <lm AT sunnyspot DOT org>",
		"Marco Lamberto",
		"16 Nov 2000",
		N_("<Save>/Color XHTML (.cxhtml)"),
		"INDEXED",
		GIMP_PLUGIN,
		nsave_args, 0,
		save_args, NULL);

	gimp_register_save_handler("file_text_colorer_save_xhtml", "cxhtml", "");
}


static void
run(gchar *name,
		gint nparams,
		GimpParam *param,
		gint *nreturn_vals,
		GimpParam **return_vals)
{
	static GimpParam values[2];
#if (GIMP_CHECK_VERSION(1, 3, 0))
	GimpRunMode run_mode;
#else
	GimpRunModeType run_mode;
#endif
	GimpPDBStatusType status = GIMP_PDB_SUCCESS;
	gint32 image_id;
	gint32 drawable_id;


	xhtml_output = g_strcasecmp(name, "file_text_colorer_save_xhtml") == 0;
	tc_status.font_size = xhtml_output ? 6 : 2;


	INIT_I18N_UI();
	gimp_ui_init("chtml", FALSE);

	run_mode = param[0].data.d_int32;

	*nreturn_vals	= 1;
	*return_vals	= values;

	values[0].type					= GIMP_PDB_STATUS;
	values[0].data.d_status	= GIMP_PDB_EXECUTION_ERROR;


	if (g_strcasecmp(name, "file_text_colorer_save") == 0 ||
		g_strcasecmp(name, "file_text_colorer_save_xhtml") == 0
	) {
		image_id		= param[1].data.d_int32;
		drawable_id	= param[2].data.d_int32;

	  switch (gimp_export_image(&image_id, &drawable_id, TITLE[xhtml_output],
			GIMP_EXPORT_CAN_HANDLE_INDEXED)) {
		case GIMP_EXPORT_CANCEL:
			status = GIMP_PDB_CANCEL;
			break;
		case GIMP_EXPORT_IGNORE:
		default:
		}
			
		if (nparams != 5 )
			status = GIMP_PDB_CALLING_ERROR;
		else {
  		gimp_get_data("plug_in_text_colorer", &tc_status);
			mw = g_new0(TCMainWindow, 1);
			mw->status = &tc_status;

			switch (run_mode) {
			case GIMP_RUN_INTERACTIVE:
				if (!save_dialog())
					status = GIMP_PDB_CANCEL;
				break;
			case GIMP_RUN_NONINTERACTIVE:
				status = GIMP_PDB_CALLING_ERROR;
				break;
			case GIMP_RUN_WITH_LAST_VALS:
				break;
			default:
				break;
			}
		}

		if (status == GIMP_PDB_SUCCESS)
			status = save_image(param[3].data.d_string, image_id, drawable_id);

		if (status == GIMP_PDB_SUCCESS)
  		gimp_set_data("plug_in_text_colorer", &tc_status, sizeof(TCStatus));
	
		g_free(mw);
	}
	values[0].data.d_status = status;
}


static gint32
save_image(gchar *filename, gint32 image_id, gint32 drawable_id)
{
	GimpDrawable *drawable;
	GimpPixelRgn pixel_rgn;
	FILE *fp, *ftxt;
	gint32 ret = GIMP_PDB_EXECUTION_ERROR;
	guchar *cmap = NULL;	/* colormap for indexed images */
	gint palsize = 0;
	guchar *buf, c;
	gint32 width;
	gint32 height;
	gint i, j, sndx, ndx, ondx;


	cmap = gimp_image_get_cmap(image_id, &palsize);
	ret = GIMP_PDB_SUCCESS;

	drawable	= gimp_drawable_get(drawable_id);
	width			= drawable->width;
	height		= drawable->height;

	if (tc_status.use_string) {
		if (strlen(tc_status.string) == 0)
			return GIMP_PDB_EXECUTION_ERROR;
	} else {
		if (strlen(tc_status.filename) == 0)
			return GIMP_PDB_EXECUTION_ERROR;

		if (!(ftxt = fopen(tc_status.filename, "rt")))
			return GIMP_PDB_EXECUTION_ERROR;
	}

	if (!(fp = fopen(filename, "wt")))
		return GIMP_PDB_EXECUTION_ERROR;

	buf = (guchar *) g_malloc(width * TC_BPP);
	gimp_pixel_rgn_init(&pixel_rgn, drawable, 0, 0, drawable->width,
		drawable->height, FALSE, FALSE);

	ondx = -1;
	fprintf(fp,
		HEAD[xhtml_output],
		g_basename(filename), width, height,
		TC_VERSION, 0x000000, tc_status.font_size);

	for (i = 0, sndx = 0; i < height; i++) {
		gimp_pixel_rgn_get_rect(&pixel_rgn, buf, 0, i, width, 1);
		for (j = 0; j < width; j++) {
			ndx = buf[j] * 3;
			if (ondx != ndx) {
				if (ondx != -1)
					fputs(CBLOCK_END[xhtml_output], fp);

				ondx = ndx;
				fprintf(fp, CBLOCK_START[xhtml_output],
					cmap[ndx], cmap[ndx + 1], cmap[ndx + 2]);
			}

			if (!tc_status.use_string) {
				for (c = -1; c <= ' ' || c > '~'; c = fgetc(ftxt)) {
					if (feof(ftxt))
						rewind(ftxt);
				}
			} else {
				for (c = -1; c <= ' ' || c > '~'; c = tc_status.string[sndx++]) {
					if (sndx >= strlen(tc_status.string))
						sndx = 0;
				}
			}
			switch (c) {
			case '<':
				fputs("&lt;", fp); break;
			case '>':
				fputs("&gt;", fp); break;
			case '&':
				fputs("&amp;", fp); break;
			default:
				fputc(c, fp);
			}
		}
		fputc('\n', fp);
	}
	fputs(TAIL[xhtml_output], fp);

	fclose(fp);

	if (!tc_status.use_string)
		fclose(ftxt);

	return GIMP_PDB_SUCCESS;
}


/** UI STUFF **/


static gboolean
save_dialog(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *rbutton;
	GSList *rbgroup = NULL;
	gchar *title;


  title = g_strconcat(_("TextColorer"), " ", TC_VERSION, NULL);
	mw->window = gimp_dialog_new(
		title, "chtml",
		gimp_standard_help_func, "filters/text_colorer.html",
		GTK_WIN_POS_MOUSE,
		FALSE, TRUE, FALSE,

		_("OK"), on_ok_clicked,
		NULL, NULL, NULL, TRUE, FALSE,
		_("Cancel"), gtk_widget_destroy,
		NULL, 1, NULL, FALSE, TRUE,

		NULL);

	gtk_signal_connect(GTK_OBJECT(mw->window), "destroy",
		GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
		
  gimp_help_init();


	mw->file_sel = gtk_file_selection_new(_("TextColorer: Select File Source"));
	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(mw->file_sel));
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(mw->file_sel)->ok_button),
		"clicked", GTK_SIGNAL_FUNC(on_file_sel_close), mw);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(mw->file_sel)->cancel_button),
		"clicked", GTK_SIGNAL_FUNC(on_file_sel_close), mw->file_sel);
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(mw->file_sel),
		mw->status->filename);
	
	
  vbox = gtk_vbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(mw->window)->vbox),
		vbox, TRUE, TRUE, 0);
  gtk_widget_show(vbox);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 2);
  gtk_widget_show(hbox);

	rbutton = gtk_radio_button_new_with_label(rbgroup, _("File"));
	rbgroup = gtk_radio_button_group(GTK_RADIO_BUTTON(rbutton));
	gtk_box_pack_start(GTK_BOX(hbox), rbutton, FALSE, TRUE, 3);
  gtk_signal_connect(GTK_OBJECT(rbutton), "toggled",
		GTK_SIGNAL_FUNC(on_file_choose), mw);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rbutton),
		!mw->status->use_string);
	gtk_widget_show(rbutton);

	mw->file_name = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), mw->file_name, TRUE, TRUE, 5);
	gtk_entry_set_text(GTK_ENTRY(mw->file_name), mw->status->filename);
	gtk_widget_set_sensitive(mw->file_name, !mw->status->use_string);
	gtk_widget_show(mw->file_name);

  mw->button_browse = gtk_button_new_with_label(_("..."));
  gtk_signal_connect(GTK_OBJECT(mw->button_browse), "clicked",
		GTK_SIGNAL_FUNC(on_browse_clicked), mw->file_sel);
  gtk_box_pack_start(GTK_BOX(hbox), mw->button_browse, FALSE, FALSE, 0);
	gtk_widget_set_sensitive(mw->button_browse, !mw->status->use_string);
  gtk_widget_show(mw->button_browse);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 2);
  gtk_widget_show(hbox);

	rbutton = gtk_radio_button_new_with_label(rbgroup, _("String"));
	gtk_box_pack_start(GTK_BOX(hbox), rbutton, FALSE, TRUE, 3);
  gtk_signal_connect(GTK_OBJECT(rbutton), "toggled",
		GTK_SIGNAL_FUNC(on_string_choose), mw);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rbutton),
		mw->status->use_string);
	gtk_widget_show(rbutton);

	mw->string = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), mw->string, TRUE, TRUE, 5);
	gtk_entry_set_text(GTK_ENTRY(mw->string), mw->status->string);
	gtk_widget_set_sensitive(mw->string, mw->status->use_string);
	gtk_widget_show(mw->string);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 2);
  gtk_widget_show(hbox);
	
  mw->adj = xhtml_output ?
		gtk_adjustment_new(mw->status->font_size, 1, 500, 1, 10, 1) :
		gtk_adjustment_new(mw->status->font_size, -7, 7, 1, 1, 1);
  mw->font_size = gtk_spin_button_new(GTK_ADJUSTMENT(mw->adj), 1, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(mw->font_size), TRUE);
  gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(mw->font_size), GTK_UPDATE_ALWAYS);
  //gtk_spin_button_set_value(GTK_SPIN_BUTTON(mw->font_size), ??? );
  gtk_box_pack_end(GTK_BOX(hbox), mw->font_size, FALSE, TRUE, 2);
  gtk_widget_show(mw->font_size);

	label = gtk_label_new(xhtml_output ?
		_("XHTML Text Font Size (px)") : _("HTML Text Font Size"));
	gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, TRUE, 3);
	gtk_widget_show(label);

	gtk_widget_show(mw->window);
	gtk_main();
	gdk_flush();

	gimp_help_free();

	return mw->ok_pressed;
}

static void 
on_ok_clicked(GtkWidget *widget, gpointer data)
{
	mw->status->font_size = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(mw->font_size));
	g_snprintf(mw->status->filename, sizeof(mw->status->filename), "%s",
		gtk_entry_get_text(GTK_ENTRY(mw->file_name)));
	g_snprintf(mw->status->string, sizeof(mw->status->string), "%s",
		gtk_entry_get_text(GTK_ENTRY(mw->string)));

	if (tc_status.use_string && strlen(tc_status.string) == 0) {
		// avoid saving with empty input source
		return;
	}

	if (!tc_status.use_string && strlen(tc_status.filename) == 0) {
		// avoid saving with empty input source
		return;
	}

	mw->ok_pressed = TRUE;
	gtk_widget_destroy(GTK_WIDGET(data));
}

static void 
on_file_sel_close(GtkWidget *widget, gpointer data)
{
	if (GTK_IS_WIDGET(data)) {
		gtk_widget_hide((GtkWidget *)data);
	} else {
		TCMainWindow *mw = (TCMainWindow *)data;
	
		gtk_entry_set_text(GTK_ENTRY(mw->file_name),
			gtk_file_selection_get_filename(GTK_FILE_SELECTION(mw->file_sel)));
		gtk_widget_hide(mw->file_sel);
	}
}

static void 
on_browse_clicked(GtkWidget *widget, gpointer data)
{
	GtkWidget *fs = data;
	
  gtk_widget_grab_focus(GTK_FILE_SELECTION(fs)->ok_button);
	gtk_widget_show(fs);
}

static void 
on_file_choose(GtkWidget *widget, gpointer data)
{
	TCMainWindow *mw = (TCMainWindow *)data;

	gtk_widget_set_sensitive(mw->file_name, TRUE);
	gtk_widget_set_sensitive(mw->button_browse, TRUE);
	gtk_widget_set_sensitive(mw->string, FALSE);
  mw->status->use_string = FALSE;
}

static void 
on_string_choose(GtkWidget *widget, gpointer data)
{
	TCMainWindow *mw = (TCMainWindow *)data;

	gtk_widget_set_sensitive(mw->file_name, FALSE);
	gtk_widget_set_sensitive(mw->button_browse, FALSE);
	gtk_widget_set_sensitive(mw->string, TRUE);
  mw->status->use_string = TRUE;
}


/* vim: set ts=2 sw=2 tw=79 ai nowrap: */
